/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.actions;

import org.experlog.util.Client;
import org.experlog.base.ESServletRequest;
import org.experlog.base.ESAction;
import org.experlog.base.ESCookie;

import org.experlog.gencode.DynHtml;
import org.experlog.gencode.ESDynHtml;
import org.experlog.util.AppConfiguration;

import javax.mail.Session;
import javax.mail.Address;
import javax.mail.Message;
import javax.mail.Multipart;
import javax.mail.BodyPart;
import javax.mail.Transport;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimeBodyPart;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.activation.DataHandler;
import java.io.File;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Properties;
import java.util.StringTokenizer;


public class SendMailMP implements ESAction {

    public boolean serverSide() { return true; }

    public boolean processRequest(Client session, ESServletRequest req,
        DynHtml page, Object info) throws Exception {
      
      String template = req.getParameter("Template");
      if(template == null && req.getParameter("Attachment") == null) {
        req.addParameter("MissingField", "Template");
        req.addParameter("ErrCode", "MISSINGFIELD");
        return false;
      }
      if(template != null && !AppConfiguration.isSecurePath(template)) {
        return false;
      }
      String from = req.getParameter("From");
      if(from == null || from.trim().length() < 5)  // 5 = length of "a@b.c"
        from = session.getAppConfiguration().get("email.From");
      if(from == null || from.trim().length() < 5) {
        req.addParameter("MissingField", "From");
        req.addParameter("ErrCode", "MISSINGFIELD");
        return false;
      }
      String to = req.getParameter("To");
      if(to != null && to.trim().length() < 5) to = null;
      String cc = req.getParameter("Cc");
      if(cc != null && cc.trim().length() < 5) cc = null;
      String bcc = req.getParameter("Bcc");
      if(bcc != null && bcc.trim().length() < 5) bcc = null;
      boolean norecipient = (to == null && cc == null && bcc == null);
      if(norecipient) {
        to = session.getAppConfiguration().get("email.ManagerEmail");
        if(to == null || to.trim().length() < 5) {
          req.addParameter("MissingField", "To");
          req.addParameter("ErrCode", "MISSINGFIELD");
          return false;
        }
      }

      String subject = req.getParameter("Subject");
      if(subject == null)
        subject = new String("(No Subject)");

      Properties prop = new Properties();
      String mailHost = session.getAppConfiguration().get("email.MailHost");
      if (mailHost != null)
        prop.put("mail.smtp.host", mailHost);

      try {
        String mailPort = session.getAppConfiguration().get("email.MailPort");
        if(mailPort != null && Integer.parseInt(mailPort) != 25)
          prop.put("mail.smtp.port", mailPort);
      } catch(NumberFormatException e) {
        // ignore
      }
      
      // SMTP Authentication parameters
      String user = session.getAppConfiguration().get("email.User");
      String passwd = "";
      if(user != null) {
        prop.put("mail.smtp.auth", "true");
        passwd = session.getAppConfiguration().get("email.Password");
        if(passwd == null) passwd = "";
      }

      Session msession = Session.getDefaultInstance(prop, null);

      MimeMessage message = new MimeMessage(msession);
      String contentType = req.getParameter("Content_type");
      if (contentType == null)
        contentType = "text/plain";

      message.setSubject(subject);

      Address address = new InternetAddress(from);
      message.setFrom(address);

      if(to != null) {
        StringTokenizer stok = new StringTokenizer(to, ",");
        while (stok.hasMoreTokens()) {
          String stto = stok.nextToken().trim();
          Address addressto = new InternetAddress(stto);
          message.addRecipient(Message.RecipientType.TO, addressto);
        }
      }
      if(cc != null) {
        StringTokenizer stok = new StringTokenizer(cc, ",");
        while (stok.hasMoreTokens()) {
          String stto = stok.nextToken().trim();
          Address addressto = new InternetAddress(stto);
          message.addRecipient(Message.RecipientType.CC, addressto);
        }
      }
      if(bcc != null) {
        StringTokenizer stok = new StringTokenizer(bcc, ",");
        while (stok.hasMoreTokens()) {
          String stto = stok.nextToken().trim();
          Address addressto = new InternetAddress(stto);
          message.addRecipient(Message.RecipientType.BCC, addressto);
        }
      }
      String replyto = req.getParameter("ReplyTo");
      if(replyto != null && replyto.length() >= 5) {
        StringTokenizer stok = new StringTokenizer(replyto, ",");
        InternetAddress addrs[] = new InternetAddress[stok.countTokens()];
        int i = 0;
        while (stok.hasMoreTokens()) {
          addrs[i++] = new InternetAddress(stok.nextToken().trim());
        }
        message.setReplyTo(addrs);
      }

      String header = req.getParameter("ReturnReceiptTo");
      if(header != null) message.addHeader("Return-Receipt-To", header);
      header = req.getParameter("ConfirmReadingTo");
      if(header != null) message.addHeader("X-Confirm-Reading-To", header);

      MimeMultipart multipart = new MimeMultipart();

      // Part one is text
      if(template != null) {
        DynHtml expander = new ESDynHtml(session, null);
        expander.setServletRequest(req);

        String root = session.getAppConfiguration().getRootDir();
        String fnm;
        if(root.endsWith(File.separator)) fnm = new String(root + template);
        else fnm = new String(root + File.separator + template);

        expander.setInputFile(new File(fnm));

        expander.addDataObject("Cookie", (ESCookie)(session.getCookie()));
        // ShopConfig is here for backward compatibility...
        // The right name is now AppConfig
        expander.addDataObject("ShopConfig", session.getAppConfiguration());
        expander.addDataObject("AppConfig", session.getAppConfiguration());

        StringWriter sw = new StringWriter();
        PrintWriter p = new PrintWriter(sw);
        expander.expand(p);
        p.flush();

        BodyPart messageBodyPart = new MimeBodyPart();
        messageBodyPart.setContent(sw.getBuffer().toString(), contentType);
        multipart.addBodyPart(messageBodyPart);
      }

      // Part two is attachment
      String filename = req.getParameter("Attachment");
      String toRemove = new String();
      if (filename != null) {
        StringTokenizer st = new StringTokenizer(filename, ",;");

        while (st.hasMoreTokens()) {
          String afile = st.nextToken();

          if (afile != null) {
            String uploaddir = session.getAppConfiguration().getUploadDir();
            if(uploaddir == null) {
              req.addParameter("MissingProperty", "app.UploadDir");
              req.addParameter("ErrCode", "MISSINGPROPERTY");
              return false;
            }
            if(!AppConfiguration.isSecurePathStrong(afile, uploaddir))
              return false;

            String nm;
            if (afile.startsWith(uploaddir))
              nm = afile;
            else
              if(uploaddir.endsWith(File.separator))
                nm = new String(uploaddir + afile);
              else
                nm = new String(uploaddir + File.separator + afile);

            toRemove += nm + ";";

            addAttachment(new File(nm), multipart);
          }
        }
      }
      message.setContent(multipart);
      message.saveChanges();
      
      if(user != null) { // SMTP authentication requested
        Transport transport = msession.getTransport("smtp");
        transport.connect (mailHost, user, passwd);
        transport.sendMessage(message, message.getAllRecipients());
        transport.close();
      } else {
        Transport.send(message);
      }

      String remove = req.getParameter("Remove");
      if ((remove != null) && (remove.compareTo("1") == 0)) {
        if (toRemove.length() > 0) {
          StringTokenizer st = new StringTokenizer(toRemove, ";");

          while (st.hasMoreTokens()) {
            deleteFileOrDir(new File(st.nextToken()));
          }
        }
      }

      return true;
    }

	private void addAttachment(File f, Multipart multipart)
	throws javax.mail.MessagingException {
		String str[] = null;
		String base = null;
		if(f.isDirectory()) {
			base = f.getPath();
			str = f.list();
		} else {
			base = f.getParent();
            if (f.exists()) {
                str = new String[1];
                str[0] = f.getName();
            }
		}

		if(str == null) return;

		for(int i=0; i<str.length; i++) {

			String nm = base + File.separator + str[i];
			File test = new File(nm);
			if(test.isDirectory()) {
				addAttachment(test, multipart); // WARNING - Recursive call
			} else {
				BodyPart messageBodyPart = new MimeBodyPart();
				DataSource source = new FileDataSource(nm);

				messageBodyPart.setDataHandler(new DataHandler(source));
				messageBodyPart.setFileName(str[i]);
				multipart.addBodyPart(messageBodyPart);
			}
		}
	}

	private void deleteFileOrDir(File f) throws java.io.IOException {
		if(f.isDirectory()) {
			File lst[] = f.listFiles();
			for(int i=0; i<lst.length; i++) {
				if(lst[i].isDirectory()) {
					deleteFileOrDir(lst[i]); // WARNING - Recursive call
				} else {
					lst[i].delete();
				}
			}
		}
        if (f.exists())
            f.delete();
	}

}

