/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.actions;

import org.experlog.util.Client;
import org.experlog.base.ESServletRequest;
import org.experlog.base.ESAction;
import org.experlog.base.ESCookie;
import org.experlog.gencode.DynHtml;
import org.experlog.gencode.ESDynHtml;

import java.io.File;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;

/**
 * Write to a file on the server disk
 * Input parameters:
 *  Template: a template file path, relative to the shop's root dir
 *   (MANDATORY)
 *  File: the output file (MANDATORY)
 *  Charset: a charset name (OPTIONAL) - eg. ISO_8859_1 or US-ASCII
 * Output parameters:
 */
public class WriteFile implements ESAction {

    public boolean serverSide() { return true; }

    public boolean processRequest(Client session, ESServletRequest req, DynHtml page, Object info) throws Exception {
	String template = req.getParameter("Template");
	if(template == null) {
	    req.addParameter("MissingField", "Template");
	    req.addParameter("ERRCODE", "MISSINGFIELD");
	    return false;
	}
	String fnam = req.getParameter("File");
	if(fnam == null) {
	    req.addParameter("MissingField", "File");
	    req.addParameter("ERRCODE", "MISSINGFIELD");
	    return false;
	}

	DynHtml expander = new ESDynHtml(session, null);
	expander.setServletRequest(req);

	String root = session.getAppConfiguration().getRootDir();
	String fnm;
	if(root.endsWith(File.separator)) fnm = new String(root + template);
	else fnm = new String(root + File.separator + template);

	expander.setInputFile(new File(fnm));

	expander.addDataObject("Cookie", (ESCookie)session.getCookie());

        // ShopConfig is here for backward compatibility...
        // The right name is now AppConfig
	expander.addDataObject("ShopConfig", session.getAppConfiguration());
	expander.addDataObject("AppConfig", session.getAppConfiguration());

	// Output file
	File tmp = new File(fnam);
	File f;
	if(tmp.isAbsolute()) {
	    f = tmp;
	} else {
	    if(root.endsWith(File.separator))
		f = new File(root + tmp);
	    else
		f = new File(root + File.separatorChar + tmp);
	}
	String charset = req.getParameter("Charset");
        PrintWriter p;
        if(charset == null)
	  p = new PrintWriter(new BufferedWriter(new FileWriter(f)));
        else
	  p = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(f), charset)));

	expander.expand(p);
	p.close();

	return true;
    }

};

